@@ -1,7 +1,7 @@
 /*
  * libopenraw - ljpegdecompressor.cpp
  *
- * Copyright (C) 2007 Hubert Figuiere
+ * Copyright (C) 2007-2016 Hubert Figuiere
  *
  * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -54,17 +54,21 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <boost/scoped_ptr.hpp>
-#include <boost/scoped_array.hpp>
+#include <fcntl.h>
+#include <algorithm>
+#include <cstdint>
+
 #include <boost/format.hpp>
 
-#include <libopenraw++/rawdata.h>
-#include "io/memstream.h"
-#include "trace.h"
-#include "rawcontainer.h"
-#include "jfifcontainer.h"
-#include "ljpegdecompressor.h"
-#include "ljpegdecompressor_priv.h"
+#include <libopenraw/consts.h>
+#include <libopenraw/debug.h>
+
+#include "rawdata.hpp"
+#include "exception.hpp"
+#include "io/stream.hpp"
+#include "trace.hpp"
+#include "ljpegdecompressor.hpp"
+#include "ljpegdecompressor_priv.hpp"
 
 namespace OpenRaw {
 
@@ -86,8 +90,7 @@ LJpegDecompressor::LJpegDecompressor(IO::Stream *stream,
       m_mcuROW1(NULL), m_mcuROW2(NULL),
       m_buf1(NULL), m_buf2(NULL),
       m_bitsLeft(0),
-      m_getBuffer(0),
-      m_output(0)
+      m_getBuffer(0)
 {
 }
 
@@ -329,7 +332,7 @@ static int32_t bmask[] = {0x0000,
  */
 void
 LJpegDecompressor::DecoderStructInit (DecompressInfo *dcPtr)
-    throw(DecodingException)
+    noexcept(false)
 {
     int16_t ci,i;
     JpegComponentInfo *compPtr;
@@ -532,7 +535,7 @@ int32_t LJpegDecompressor::get_bit()
 }
 
 
-inline 
+inline
 int32_t LJpegDecompressor::readBits(IO::Stream * s, uint16_t nbits)
 {
     if (m_bitsLeft < nbits) {
@@ -558,22 +561,22 @@ int32_t LJpegDecompressor::readBits(IO::Stream * s, uint16_t nbits)
  *
  *--------------------------------------------------------------
  */
-inline void 
+inline void
 LJpegDecompressor::PmPutRow(MCU* RowBuf, int32_t numComp, int32_t numCol, int32_t Pt)
-{ 
+{
     // TODO this might be wrong in 8 bits...
     // original code was using putc which *i think* was a problem for
     // 16bpp
-	int32_t comp;
+    int32_t comp;
     int32_t col;
     uint16_t v;
-							
-	for (col = 0; col < numCol; col++) {	
-		for (comp = 0; comp < numComp; comp++) {
+
+    for (col = 0; col < numCol; col++) {
+        for (comp = 0; comp < numComp; comp++) {
             v = RowBuf[col][comp]<<Pt;
             m_output->append(v);
         }
-	}
+    }
 //			m_output->nextRow();
 }
 
@@ -648,19 +651,23 @@ LJpegDecompressor::HuffDecode(HuffmanTable *htbl)
  *
  *--------------------------------------------------------------
  */
-static int32_t extendTest[16] =	/* entry n is 2**(n-1) */
+static const int32_t extendTest[16] =	/* entry n is 2**(n-1) */
 {0, 0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0020, 0x0040, 0x0080,
  0x0100, 0x0200, 0x0400, 0x0800, 0x1000, 0x2000, 0x4000};
 
-static int32_t extendOffset[16] =	/* entry n is (-1 << n) + 1 */
-{0, ((-1) << 1) + 1, ((-1) << 2) + 1, ((-1) << 3) + 1, ((-1) << 4) + 1,
- ((-1) << 5) + 1, ((-1) << 6) + 1, ((-1) << 7) + 1, ((-1) << 8) + 1,
- ((-1) << 9) + 1, ((-1) << 10) + 1, ((-1) << 11) + 1, ((-1) << 12) + 1,
- ((-1) << 13) + 1, ((-1) << 14) + 1, ((-1) << 15) + 1};
+// we can't bitshift -1. So we use 0xffffffff as a value.
+// gcc complain about it otherwise.
+#define EXTEND(n) (int32_t)(0xffffffff << n) + 1
+static const int32_t extendOffset[16] =	/* entry n is (-1 << n) + 1 */
+{
+    0, EXTEND(1), EXTEND(2), EXTEND(3),
+    EXTEND(4), EXTEND(5), EXTEND(6), EXTEND(7),
+    EXTEND(8), EXTEND(9), EXTEND(10), EXTEND(11),
+    EXTEND(12), EXTEND(13), EXTEND(14), EXTEND(15)
+};
 
-		
 inline
-void HuffExtend(int32_t & x, int32_t s) 
+void HuffExtend(int32_t & x, int32_t s) noexcept
 {
     if ((x) < extendTest[s]) {
         (x) += extendOffset[s];
@@ -685,7 +692,7 @@ void HuffExtend(int32_t & x, int32_t s)
  */
 void
 LJpegDecompressor::HuffDecoderInit (DecompressInfo *dcPtr)
-    throw(DecodingException)
+    noexcept(false)
 {
     int16_t ci;
     JpegComponentInfo *compptr;
@@ -737,7 +744,7 @@ LJpegDecompressor::HuffDecoderInit (DecompressInfo *dcPtr)
  */
 void
 LJpegDecompressor::ProcessRestart (DecompressInfo *dcPtr)
-    throw(DecodingException)
+    noexcept(false)
 {
     int32_t c, nbytes;
 
@@ -1059,7 +1066,7 @@ static inline void SkipVariable(IO::Stream * s)
  */
 void
 LJpegDecompressor::GetDht (DecompressInfo *dcPtr)
-    throw(DecodingException)
+    noexcept(false)
 {
     int32_t length;
     int32_t i, index, count;
@@ -1118,7 +1125,7 @@ LJpegDecompressor::GetDht (DecompressInfo *dcPtr)
  */
 void
 LJpegDecompressor::GetDri(DecompressInfo *dcPtr)
-    throw(DecodingException)
+    noexcept(false)
 {
     if (Get2bytes(m_stream) != 4) {
         throw DecodingException("Bogus length in DRI");
@@ -1168,7 +1175,8 @@ static void	GetApp0(IO::Stream *s)
  *--------------------------------------------------------------
  */
 void
-LJpegDecompressor::GetSof(DecompressInfo *dcPtr) throw(DecodingException)
+LJpegDecompressor::GetSof(DecompressInfo *dcPtr)
+    noexcept(false)
 {
     int32_t length;
     int16_t ci;
@@ -1234,7 +1242,7 @@ LJpegDecompressor::GetSof(DecompressInfo *dcPtr) throw(DecodingException)
  */
 void
 LJpegDecompressor::GetSos (DecompressInfo *dcPtr)
-    throw(DecodingException)
+    noexcept(false)
 {
     int32_t length;
     int32_t i;
@@ -1398,7 +1406,7 @@ LJpegDecompressor::ProcessTables (DecompressInfo *dcPtr)
             break;
 
         case M_DQT:
-            Trace(WARNING) << "Not a lossless JPEG file.\n";
+            LOGWARN("Not a lossless JPEG file.\n");
             break;
 
         case M_DRI:
@@ -1418,8 +1426,7 @@ LJpegDecompressor::ProcessTables (DecompressInfo *dcPtr)
         case M_RST6:
         case M_RST7:
         case M_TEM:
-            Trace(WARNING) << str(boost::format("Warning: unexpected "
-                                                "marker 0x%1%") % c);
+            LOGWARN("Warning: unexpected marker 0x%x", c);
             break;
 
         default:		/* must be DNL, DHP, EXP, APPn, JPGn, COM,
@@ -1448,10 +1455,10 @@ LJpegDecompressor::ProcessTables (DecompressInfo *dcPtr)
  */
 void
 LJpegDecompressor::ReadFileHeader (DecompressInfo *dcPtr)
-    throw(DecodingException)
+    noexcept(false)
 {
     int c, c2;
-			
+
     /*
      * Demand an SOI marker at the start of the file --- otherwise it's
      * probably not a JPEG file at all.
@@ -1463,28 +1470,27 @@ LJpegDecompressor::ReadFileHeader (DecompressInfo *dcPtr)
                                                   "marker is %1% %2%\n")
                                     % c % c2));
     }
-			
+
     GetSoi (dcPtr);		/* OK, process SOI */
-			
+
     /*
      * Process markers until SOF
      */
     c = ProcessTables (dcPtr);
-			
+
     switch (c) {
     case M_SOF0:
     case M_SOF1:
     case M_SOF3:
         GetSof(dcPtr);
         break;
-				
+
     default:
-        Trace(WARNING) << str(boost::format("Unsupported SOF marker "
-                                            "type 0x%1%\n") % c);
+        LOGWARN("Unsupported SOF marker type 0x%x\n", c);
         break;
     }
 }
-			
+
 /*
  *--------------------------------------------------------------
  *
@@ -1527,25 +1533,21 @@ LJpegDecompressor::ReadScanHeader (DecompressInfo *dcPtr)
 }
 
 
-RawData *LJpegDecompressor::decompress(RawData *bitmap)
+RawDataPtr LJpegDecompressor::decompress()
 {
     DecompressInfo dcInfo;
     try {
         ReadFileHeader(&dcInfo); 
         ReadScanHeader (&dcInfo);
 
-        if(bitmap == NULL)
-        {
-            bitmap = new RawData();
-        }
-        m_output = bitmap;
-        bitmap->setDataType(OR_DATA_TYPE_CFA);
+        m_output = RawDataPtr(new RawData);
+        m_output->setDataType(OR_DATA_TYPE_RAW);
         uint32_t bpc = dcInfo.dataPrecision;
 
-        bitmap->setBpc(bpc);
-        bitmap->setMax((1 << bpc) - 1);
+        m_output->setBpc(bpc);
+        m_output->setWhiteLevel((1 << bpc) - 1);
         /*uint16_t *dataPtr = (uint16_t*)*/
-        bitmap->allocData(dcInfo.imageWidth
+        m_output->allocData(dcInfo.imageWidth
                           * sizeof(uint16_t) 
                           * dcInfo.imageHeight
                           * dcInfo.numComponents);
@@ -1558,8 +1560,8 @@ RawData *LJpegDecompressor::decompress(RawData *bitmap)
          * @todo check that this is valid with DNG too.
          */ 
         uint32_t width = dcInfo.imageWidth * dcInfo.numComponents;
-        bitmap->setDimensions(width, dcInfo.imageHeight);
-        bitmap->setSlices(m_slices);
+        m_output->setDimensions(width, dcInfo.imageHeight);
+        m_output->setSlices(m_slices);
         DecoderStructInit(&dcInfo);
         HuffDecoderInit(&dcInfo);
         DecodeImage(&dcInfo);
@@ -1569,8 +1571,7 @@ RawData *LJpegDecompressor::decompress(RawData *bitmap)
     {
         Trace(ERROR) << "Decompression error\n";
     }
-    m_output = NULL;
-    return bitmap;
+    return std::move(m_output);
 }
 
 }
