@@ -1,7 +1,7 @@
 /*
  * libopenraw - testsuite.cpp
  *
- * Copyright (C) 2008-2009 Hubert Figuiere
+ * Copyright (C) 2008-2016 Hubert Figuiere
  * Copyright (C) 2008 Novell, Inc.
  *
  * This library is free software: you can redistribute it and/or
@@ -26,106 +26,151 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <stddef.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <cstdint>
 #include <unistd.h>
 #include <errno.h>
+#include <getopt.h>
 
-#include <libxml/xmlreader.h>
+#include <libxml/parser.h>
+#include <libxml/xmlstring.h>
 #include <libxml/tree.h>
 #include <libxml/xmlsave.h>
 
+#include <algorithm>
 #include <iostream>
-#include <numeric>
-#include <stack>
 #include <string>
 #include <utility>
 #include <vector>
-#include <boost/shared_ptr.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/scoped_array.hpp>
-#include <boost/bind.hpp>
+
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/algorithm/string/classification.hpp>
 #include <boost/crc.hpp>      // for boost::crc_basic, boost::crc_optimal
 
-#include <libopenraw++/rawfile.h>
-#include <libopenraw++/rawdata.h>
-#include <libopenraw++/thumbnail.h>
-#include <libopenraw++/bitmapdata.h>
-#include "io/file.h"
+#define IN_TESTSUITE
+#include <libopenraw/debug.h>
+#include <libopenraw/consts.h>
+#include <libopenraw/metadata.h>
+
+#include "rawfile.hpp"
+#include "rawdata.hpp"
+#include "thumbnail.hpp"
+#include "bitmapdata.hpp"
+#include "metavalue.hpp"
+#include "cfapattern.hpp"
 
 #include "xmlhandler.h"
 #include "testsuite.h"
 #include "testsuitehandler.h"
 #include "testsuitetags.h"
 
+// Internal stuff. Because we can.
+#include "io/file.hpp"
+#include "ifdfile.hpp"
+#include "ifddir.hpp"
+#include "makernotedir.hpp"
+
 using OpenRaw::RawFile;
 using OpenRaw::BitmapData;
 using OpenRaw::RawData;
 using OpenRaw::Thumbnail;
 
+using std::unique_ptr;
 
-#define RETURN_TEST_EQUALS(a,b,expected)                                \
-    {                                                                   \
-        bool _success = (a == b);					\
-        if(!_success) {                                                 \
-            fprintf(stderr, "FAILED: %s on equality. found %d , "       \
-                    "expected '%s'\n",                                  \
-                    __FUNCTION__, a, expected.c_str());                 \
-        }                                                               \
-        return _success;                                                \
+#define RETURN_TEST_EQUALS(a,b) \
+    {                               \
+        bool _success = (a == b);   \
+        if(!_success) {             \
+            fprintf(stderr, "FAILED: %s on equality. found '%s', "  \
+                    "expected '%s'\n",                       \
+                    __FUNCTION__, a.c_str(), b.c_str());     \
+        }                           \
+        return _success;            \
+    }
+
+#define RETURN_TEST_EQUALS_N(a,b) \
+    {                               \
+        bool _success = (a == b);   \
+        if(!_success) {             \
+            fprintf(stderr, "FAILED: %s on equality. found %ld, "  \
+                    "expected '%ld'\n",                             \
+                    __FUNCTION__, (long)a, (long)b);                \
+        }                           \
+        return _success;            \
     }
 
 
-#define RETURN_TEST(test,expected)                                      \
-    {                                                                   \
-        bool _success = (test);                                         \
-        if(!_success) {                                                 \
-            fprintf(stderr, "FAILED: %s on '%s', expected '%s'\n",	\
-                    __FUNCTION__, #test, expected.c_str());		\
-        }                                                               \
-        return _success;                                                \
+// a and b are strings. b is the expected value
+// success is return. Set to true if it is successful
+#define CHECK_TEST_EQUALS(a,b,success)  \
+    {                                   \
+        success = (a == b);             \
+        if(!success) {                  \
+            fprintf(stderr, "FAILED: %s on equality with '%s', expected '%s'\n", \
+                    __FUNCTION__, a.c_str(), b.c_str());     \
+        }                                                    \
     }
 
-#define RETURN_FAIL(message,expected)                                   \
-    {                                                                   \
-        fprintf(stderr, "FAILED: %s with '%s', expected '%s'\n",	\
-                __FUNCTION__, message, expected.c_str());		\
-        return false;                                                   \
+// a and b are integers. b is the expected value
+// success is return. Set to true if it is successful
+#define CHECK_TEST_EQUALS_N(a,b,success) \
+    {                                    \
+        success = (a == b);              \
+        if(!success) {                   \
+            fprintf(stderr, "FAILED: %s on equality with %ld, expected %ld\n",	\
+                    __FUNCTION__, (long)a, (long)b);   \
+        }                                \
+    }
+
+#define RETURN_TEST(test,expected)       \
+    {                                    \
+        bool _success = (test);          \
+        if(!_success) {                  \
+            fprintf(stderr, "FAILED: %s on '%s', expected '%s'\n", \
+                    __FUNCTION__, #test, expected.c_str()); \
+        }                                \
+        return _success;                 \
+    }
+
+#define RETURN_FAIL(message,expected)    \
+    {                                    \
+        fprintf(stderr, "FAILED: %s with '%s', expected '%s'\n", \
+                __FUNCTION__, message, expected.c_str()); \
+        return false;                    \
     }
 
 namespace {
-bool equalCfaPattern(const std::string & result, RawData::CfaPattern t)
+std::string cfaPatternToString(::or_cfa_pattern t)
 {
-    bool equal = false;
     switch(t) {
     case OR_CFA_PATTERN_NONE:
-        equal = (result == "NONE");
-        break;
+        return "NONE";
+
     case OR_CFA_PATTERN_NON_RGB22:
-        equal = (result == "NON_RGB22");
-        break;
+        return "NON_RGB22";
+
     case OR_CFA_PATTERN_RGGB:
-        equal = (result == "RGGB");
-        break;
+        return "RGGB";
+
     case OR_CFA_PATTERN_GBRG:
-        equal = (result == "GBRG");
-        break;
+        return "GBRG";
+
     case OR_CFA_PATTERN_BGGR:
-        equal = (result == "BGGR");
-        break;
+        return "BGGR";
+
     case OR_CFA_PATTERN_GRBG:
-        equal = (result == "GRBG");
-        break;
+        return "GRBG";
+
     default:
         break;
     }
-    return equal;
+    return "";
 }
-	
+
 
 bool equalDataType(const std::string & result, BitmapData::DataType t)
 {
@@ -143,11 +188,11 @@ bool equalDataType(const std::string & result, BitmapData::DataType t)
     case OR_DATA_TYPE_PNG:
         equal = (result == "PNG");
         break;
-    case OR_DATA_TYPE_CFA:
-        equal = (result == "CFA");
+    case OR_DATA_TYPE_RAW:
+        equal = (result == "RAW");
         break;
-    case OR_DATA_TYPE_COMPRESSED_CFA:
-        equal = (result == "COMP_CFA");
+    case OR_DATA_TYPE_COMPRESSED_RAW:
+        equal = (result == "COMP_RAW");
         break;
     default:
         break;
@@ -158,16 +203,14 @@ bool equalDataType(const std::string & result, BitmapData::DataType t)
 }
 
 Test::Test()
-    : m_rawfile(NULL),
-      m_rawdata(NULL),
+    : m_rawfile(nullptr),
+      m_rawdata(nullptr),
       m_total(0), m_success(0), m_failure(0)
 {
 }
 
 Test::~Test()
 {
-    delete m_rawfile;
-    delete m_rawdata;
 }
 
 bool Test::testRawType(const std::string & result)
@@ -179,14 +222,14 @@ bool Test::testRawType(const std::string & result)
     OpenRaw::IO::File f(m_file.c_str());
     ::or_error err = f.open();
     if(err != OR_ERROR_NONE) {
-        RETURN_FAIL("failed to open", 
+        RETURN_FAIL("failed to open",
                     boost::lexical_cast<std::string>(err));
     }
     off_t len = f.filesize();
-    boost::scoped_array<uint8_t> buff(new uint8_t[len]);
+    unique_ptr<uint8_t[]> buff(new uint8_t[len]);
     int res = f.read(buff.get(), len);
     if(res == len) {
-        boost::scoped_ptr<RawFile> r2(RawFile::newRawFileFromMemory(buff.get(), len));
+        unique_ptr<RawFile> r2(RawFile::newRawFileFromMemory(buff.get(), len));
         if(!r2) {
             RETURN_FAIL("failed to load from memory", std::string());
         }
@@ -196,7 +239,7 @@ bool Test::testRawType(const std::string & result)
         }
     }
     else {
-        RETURN_FAIL("failed to load into buffer", 
+        RETURN_FAIL("failed to load into buffer",
                     boost::lexical_cast<std::string>(res));
     }
 
@@ -228,6 +271,15 @@ bool Test::testRawType(const std::string & result)
     case OR_RAWFILE_TYPE_ERF:
         RETURN_TEST(result == "ERF", result);
         break;
+    case OR_RAWFILE_TYPE_NRW:
+        RETURN_TEST(result == "NRW", result);
+        break;
+    case OR_RAWFILE_TYPE_RW2:
+        RETURN_TEST(result == "RW2", result);
+        break;
+    case OR_RAWFILE_TYPE_RAF:
+        RETURN_TEST(result == "RAF", result);
+        break;
     default:
         break;
     }
@@ -236,7 +288,8 @@ bool Test::testRawType(const std::string & result)
 
 bool Test::testRawTypeId(const std::string & result)
 {
-    RETURN_TEST(m_rawfile->typeId() == boost::lexical_cast<uint32_t>(result), result);
+    RETURN_TEST_EQUALS_N(m_rawfile->typeId(),
+                         boost::lexical_cast<uint32_t>(result));
 }
 
 
@@ -245,7 +298,7 @@ bool Test::testThumbNum(const std::string & result)
     const std::vector<uint32_t> & thumbs = m_rawfile->listThumbnailSizes();
     int num = thumbs.size();
     try {
-        RETURN_TEST(num == boost::lexical_cast<int>(result), result);
+        RETURN_TEST_EQUALS_N(num, boost::lexical_cast<int>(result));
     }
     catch(...)
     {
@@ -262,32 +315,31 @@ bool Test::testThumbSizes(const std::string & result)
         RETURN_FAIL("mismatch number of elements", result);
     }
     std::vector<uint32_t> v2;
-    for(std::vector< std::string >::iterator iter = v.begin();
-        iter != v.end(); iter++) 
+    for(const auto & s : v)
     {
         try {
-            v2.push_back(boost::lexical_cast<uint32_t>(*iter));
+            v2.push_back(boost::lexical_cast<uint32_t>(s));
         }
         catch(...)
         {
             RETURN_FAIL("conversion failed", result);
         }
     }
-    RETURN_TEST(std::equal(thumbs.begin(), thumbs.end(), v2.begin()), result);
+    RETURN_TEST(std::equal(thumbs.cbegin(), thumbs.cend(), v2.cbegin()), result);
 }
 
 bool Test::testThumbFormats(const std::string & result)
 {
     bool success = true;
-    std::vector<uint32_t> thumbs = m_rawfile->listThumbnailSizes();
+    auto thumbs = m_rawfile->listThumbnailSizes();
     std::vector< std::string > v;
     boost::split(v, result, boost::is_any_of(" "));
-    std::vector< std::string >::iterator result_iter = v.begin();
+    auto result_iter = v.cbegin();
     if(v.size() != thumbs.size()) {
         RETURN_FAIL("mismatch number of elements", result);
     }
-    for(std::vector<uint32_t>::iterator thumbs_iter = thumbs.begin();
-        thumbs_iter != thumbs.end(); thumbs_iter++, result_iter++) 
+    for(auto thumbs_iter = thumbs.cbegin();
+        thumbs_iter != thumbs.cend(); thumbs_iter++, result_iter++)
     {
         Thumbnail t;
         m_rawfile->getThumbnail(*thumbs_iter, t);
@@ -299,15 +351,15 @@ bool Test::testThumbFormats(const std::string & result)
 bool Test::testThumbDataSizes(const std::string & result)
 {
     bool success = true;
-    std::vector<uint32_t> thumbs = m_rawfile->listThumbnailSizes();
+    auto thumbs = m_rawfile->listThumbnailSizes();
     std::vector< std::string > v;
     boost::split(v, result, boost::is_any_of(" "));
-    std::vector< std::string >::iterator result_iter = v.begin();
+    auto result_iter = v.cbegin();
     if(v.size() != thumbs.size()) {
         RETURN_FAIL("mismatch number of elements", result);
     }
-    for(std::vector<uint32_t>::iterator thumbs_iter = thumbs.begin();
-        thumbs_iter != thumbs.end(); thumbs_iter++, result_iter++) 
+    for(auto thumbs_iter = thumbs.cbegin(); thumbs_iter != thumbs.cend();
+        thumbs_iter++, result_iter++)
     {
         Thumbnail t;
         m_rawfile->getThumbnail(*thumbs_iter, t);
@@ -322,23 +374,62 @@ bool Test::testThumbDataSizes(const std::string & result)
 }
 
 namespace {
-RawData * loadRawData(RawFile * file)
+uint32_t computeCrc(const Thumbnail * thumb)
 {
-    RawData *rawdata = new RawData();
+    boost::crc_optimal<16, 0x1021, 0xFFFF, 0, false, false>  crc_ccitt2;
+
+    const uint8_t * data = static_cast<uint8_t *>(thumb->data());
+    size_t data_len = thumb->size();
+    crc_ccitt2 = std::for_each( data, data + data_len, crc_ccitt2 );
+    return crc_ccitt2();
+}
+}
+
+bool Test::testThumbMd5(const std::string & result)
+{
+    bool success = true;
+    auto thumbs = m_rawfile->listThumbnailSizes();
+    std::vector< std::string > v;
+    boost::split(v, result, boost::is_any_of(" "));
+    auto result_iter = v.cbegin();
+    if(v.size() != thumbs.size()) {
+        RETURN_FAIL("mismatch number of elements", result);
+    }
+    for(auto thumbs_iter = thumbs.cbegin(); thumbs_iter != thumbs.cend();
+        thumbs_iter++, result_iter++)
+    {
+        Thumbnail t;
+        m_rawfile->getThumbnail(*thumbs_iter, t);
+        try {
+            bool succ = false;
+            uint32_t crc = computeCrc(&t);
+            CHECK_TEST_EQUALS(boost::lexical_cast<std::string>(crc), (*result_iter), succ);
+            success &= succ;
+        }
+        catch(...) {
+            RETURN_FAIL("conversion failed", result);
+        }
+    }
+    RETURN_TEST(success, result);
+}
+
+namespace {
+unique_ptr<RawData> loadRawData(const unique_ptr<RawFile> & file)
+{
+    unique_ptr<RawData> rawdata(new RawData());
     ::or_error err;
     err = file->getRawData(*rawdata, OR_OPTIONS_NONE);
     if(OR_ERROR_NONE != err) {
-        delete rawdata; 
-        rawdata = NULL;
+        rawdata.reset();
     }
     return rawdata;
 }
 
-uint32_t computeCrc(const RawData * rawdata)
+uint32_t computeCrc(const unique_ptr<RawData> & rawdata)
 {
-    boost::crc_optimal<8, 0x1021, 0xFFFF, 0, false, false>  crc_ccitt2;
-		
-    const uint8_t * data = static_cast<uint8_t *>(rawdata->data());
+    boost::crc_optimal<16, 0x1021, 0xFFFF, 0, false, false>  crc_ccitt2;
+
+    auto data = static_cast<const uint8_t *>(rawdata->data());
     size_t data_len = rawdata->size();
     crc_ccitt2 = std::for_each( data, data + data_len, crc_ccitt2 );
     return crc_ccitt2();
@@ -367,12 +458,11 @@ bool Test::testRawDataSize(const std::string & result)
         }
     }
     try {
-        RETURN_TEST(boost::lexical_cast<uint32_t>(result) == m_rawdata->size(),
-                    result);
+        RETURN_TEST_EQUALS_N(m_rawdata->size(), boost::lexical_cast<uint32_t>(result));
     }
     catch(...) {
     }
-    RETURN_FAIL("conversion failed", result);		
+    RETURN_FAIL("conversion failed", result);
 }
 
 bool Test::testRawDataDimensions(const std::string & result)
@@ -389,17 +479,51 @@ bool Test::testRawDataDimensions(const std::string & result)
         RETURN_FAIL("mismatch number of elements from expected result", result);
     }
     uint32_t x, y;
+    bool success = true;
     try {
         x = boost::lexical_cast<uint32_t>(v[0]);
         y = boost::lexical_cast<uint32_t>(v[1]);
+        bool succ = false;
+        CHECK_TEST_EQUALS_N(m_rawdata->width(), x, succ);
+        success |= succ;
+        CHECK_TEST_EQUALS_N(m_rawdata->height(), y,  succ);
+        success |= succ;
     }
     catch(...)
     {
         RETURN_FAIL("conversion failed", result);
     }
-    RETURN_TEST(x == m_rawdata->x() && y == m_rawdata->y(), result)
-        }
+    RETURN_TEST(success, result)
+}
 
+bool Test::testRawDataRoi(const std::string & result)
+{
+    if(m_rawdata == NULL) {
+        m_rawdata = loadRawData(m_rawfile);
+        if(m_rawdata == NULL) {
+            RETURN_FAIL("failed to get rawData", result);
+        }
+    }
+    std::vector< std::string > v;
+    boost::split(v, result, boost::is_any_of(" "));
+    if(v.size() != 4) {
+        RETURN_FAIL("mismatch number of elements from expected result", result);
+    }
+    uint32_t x, y, w, h;
+    try {
+        x = boost::lexical_cast<uint32_t>(v[0]);
+        y = boost::lexical_cast<uint32_t>(v[1]);
+        w = boost::lexical_cast<uint32_t>(v[2]);
+        h = boost::lexical_cast<uint32_t>(v[3]);
+    }
+    catch(...)
+    {
+        RETURN_FAIL("conversion failed", result);
+    }
+    RETURN_TEST(x == m_rawdata->roi_x() && y == m_rawdata->roi_y()
+        && w == m_rawdata->roi_width() && h == m_rawdata->roi_height(),
+        result)
+}
 
 bool Test::testRawCfaPattern(const std::string & result)
 {
@@ -409,7 +533,11 @@ bool Test::testRawCfaPattern(const std::string & result)
             RETURN_FAIL("failed to get rawData", result);
         }
     }
-    RETURN_TEST(equalCfaPattern(result, m_rawdata->cfaPattern()), result);
+    bool succ = false;
+    CHECK_TEST_EQUALS(cfaPatternToString(
+                          m_rawdata->cfaPattern()->patternType()),
+                      result, succ);
+    return succ;
 }
 
 bool Test::testRawMinValue(const std::string & result)
@@ -428,7 +556,7 @@ bool Test::testRawMinValue(const std::string & result)
     {
         RETURN_FAIL("conversion failed", result);
     }
-    RETURN_TEST(m_rawdata->min() == expected, result);
+    RETURN_TEST_EQUALS_N(m_rawdata->blackLevel(), expected);
 }
 
 
@@ -448,7 +576,7 @@ bool Test::testRawMaxValue(const std::string & result)
     {
         RETURN_FAIL("conversion failed", result);
     }
-    RETURN_TEST(m_rawdata->max() == expected, result);
+    RETURN_TEST_EQUALS_N(m_rawdata->whiteLevel(), expected);
 }
 
 bool Test::testRawMd5(const std::string & result)
@@ -463,14 +591,14 @@ bool Test::testRawMd5(const std::string & result)
     uint32_t crc = computeCrc(m_rawdata);
 
     uint32_t expected = 0;
-    try { 
+    try {
         expected = boost::lexical_cast<uint32_t>(result);
     }
     catch(...)
     {
         RETURN_FAIL("conversion failed", result);
     }
-    RETURN_TEST_EQUALS(crc, expected, result);
+    RETURN_TEST_EQUALS_N(crc, expected);
 }
 
 
@@ -489,7 +617,62 @@ bool Test::testRawDecompressedMd5(const std::string & result)
 bool Test::testMetaOrientation(const std::string & result)
 {
     int32_t orientation = m_rawfile->getOrientation();
-    RETURN_TEST_EQUALS(orientation, boost::lexical_cast<int32_t>(result), result);
+    RETURN_TEST_EQUALS_N(orientation, boost::lexical_cast<int32_t>(result));
+}
+
+
+bool Test::testExifString(int32_t meta_index, const std::string & result)
+{
+    auto val = m_rawfile->getMetaValue(meta_index);
+    if (val) {
+        //
+        auto stringVal = val->getString(0);
+        RETURN_TEST_EQUALS(stringVal, result);
+    }
+    RETURN_FAIL("meta data not found", result);
+}
+
+bool Test::testMakerNoteCount(const std::string & result)
+{
+    try {
+        OpenRaw::Internals::IfdFile & ifd_file =
+            dynamic_cast<OpenRaw::Internals::IfdFile &>(*m_rawfile.get());
+        auto exif = ifd_file.exifIfd();
+        auto maker_note = exif->getMakerNoteIfd();
+        if (!maker_note) {
+            RETURN_FAIL("no maker not found", result);
+        }
+        RETURN_TEST_EQUALS_N(maker_note->numTags(),
+                             boost::lexical_cast<int32_t>(result));
+    }
+    catch(const std::bad_cast & e) {
+        RETURN_FAIL("not an IFD file", result);
+    }
+    catch(...) {
+        RETURN_FAIL("unknown exception", result);
+    }
+}
+
+bool Test::testMakerNoteId(const std::string & result)
+{
+    try {
+        OpenRaw::Internals::IfdFile& ifd_file =
+            dynamic_cast<OpenRaw::Internals::IfdFile&>(*m_rawfile.get());
+        auto exif = ifd_file.exifIfd();
+        auto maker_note =
+            std::dynamic_pointer_cast<OpenRaw::Internals::MakerNoteDir>(
+                exif->getMakerNoteIfd());
+        if (!maker_note) {
+            RETURN_FAIL("no maker not found", result);
+        }
+        RETURN_TEST_EQUALS(maker_note->getId(), result);
+    }
+    catch(const std::bad_cast & e) {
+        RETURN_FAIL("not an IFD file", result);
+    }
+    catch(...) {
+        RETURN_FAIL("unknown exception", result);
+    }
 }
 
 
@@ -507,61 +690,80 @@ int Test::run()
         fprintf(stderr, "File not found, skipping. (%d)\n", errno);
         return 0;
     }
-    m_rawfile = RawFile::newRawFile(m_file.c_str());
+    m_rawfile.reset(RawFile::newRawFile(m_file.c_str()));
 
     if(m_rawfile == NULL) {
         RETURN_FAIL("m_rawfile == NULL", std::string("not NULL"));
     }
-	
-    std::map<int, std::string>::const_iterator iter;
-    for(iter = m_results.begin(); iter != m_results.end(); iter++) {
+
+    for(const auto & elem : m_results) {
         bool pass = false;
-        switch(iter->first)
+        switch(elem.first)
         {
         case XML_rawType:
-            pass = testRawType(iter->second);
+            pass = testRawType(elem.second);
             break;
         case XML_rawTypeId:
-            pass = testRawTypeId(iter->second);
+            pass = testRawTypeId(elem.second);
             break;
         case XML_thumbNum:
-            pass = testThumbNum(iter->second);
+            pass = testThumbNum(elem.second);
             break;
         case XML_thumbSizes:
-            pass = testThumbSizes(iter->second);
+            pass = testThumbSizes(elem.second);
             break;
         case XML_thumbFormats:
-            pass = testThumbFormats(iter->second);
+            pass = testThumbFormats(elem.second);
             break;
         case XML_thumbDataSizes:
-            pass = testThumbDataSizes(iter->second);
+            pass = testThumbDataSizes(elem.second);
+            break;
+        case XML_thumbMd5:
+            pass = testThumbMd5(elem.second);
             break;
         case XML_rawDataType:
-            pass = testRawDataType(iter->second);
+            pass = testRawDataType(elem.second);
             break;
         case XML_rawDataSize:
-            pass = testRawDataSize(iter->second);
+            pass = testRawDataSize(elem.second);
             break;
         case XML_rawDataDimensions:
-            pass = testRawDataDimensions(iter->second);
+            pass = testRawDataDimensions(elem.second);
+            break;
+        case XML_rawDataRoi:
+            pass = testRawDataRoi(elem.second);
             break;
         case XML_rawCfaPattern:
-            pass = testRawCfaPattern(iter->second);
+            pass = testRawCfaPattern(elem.second);
             break;
         case XML_rawMinValue:
-            pass = testRawMinValue(iter->second);
+            pass = testRawMinValue(elem.second);
             break;
         case XML_rawMaxValue:
-            pass = testRawMaxValue(iter->second);
+            pass = testRawMaxValue(elem.second);
             break;
         case XML_rawMd5:
-            pass = testRawMd5(iter->second);
+            pass = testRawMd5(elem.second);
             break;
         case XML_rawDecompressedMd5:
-            pass = testRawDecompressedMd5(iter->second);
+            pass = testRawDecompressedMd5(elem.second);
             break;
         case XML_metaOrientation:
-            pass = testMetaOrientation(iter->second);
+            pass = testMetaOrientation(elem.second);
+            break;
+        case XML_exifMake:
+            pass = testExifString(META_NS_TIFF | EXIF_TAG_MAKE,
+                                  elem.second);
+            break;
+        case XML_exifModel:
+            pass = testExifString(META_NS_TIFF | EXIF_TAG_MODEL,
+                                  elem.second);
+            break;
+        case XML_makerNoteCount:
+            pass = testMakerNoteCount(elem.second);
+            break;
+        case XML_makerNoteId:
+            pass = testMakerNoteId(elem.second);
             break;
         default:
             pass = false;
@@ -592,10 +794,8 @@ void Test::merge(const Test::Ptr & t)
     }
     // the results for t invariably replace the
     // existing on in this.
-    std::map<int, std::string>::iterator iter;
-    for(iter = t->m_results.begin(); 
-        iter != t->m_results.end(); ++iter) {
-        m_results[iter->first] = iter->second;
+    for(const auto & elem : t->m_results) {
+        m_results[elem.first] = elem.second;
     }
 }
 
@@ -605,12 +805,11 @@ TestSuite::TestSuite()
 }
 
 
-void TestSuite::add_test(const Test::Ptr & t)
+void TestSuite::add_test(Test::Ptr && t)
 {
-    std::map<std::string, Test::Ptr>::iterator iter 
-        = m_tests.find(t->name());
+    auto iter = m_tests.find(t->name());
     if(iter == m_tests.end()) {
-        m_tests.insert(make_pair(t->name(), t));
+        m_tests.insert(std::make_pair(t->name(), std::move(t)));
     }
     else {
         iter->second->merge(t);
@@ -621,7 +820,7 @@ void TestSuite::add_test(const Test::Ptr & t)
 int TestSuite::load_tests(const char * testsuite_file)
 {
     xml::HandlerPtr handler(new TestSuiteHandler(testsuite_file, this));
-	
+
     bool has_data = false;
 
     has_data = handler->process();
@@ -632,7 +831,7 @@ int TestSuite::load_tests(const char * testsuite_file)
 int TestSuite::load_overrides(const std::string & overrides_file)
 {
     xml::HandlerPtr handler(new TestSuiteHandler(overrides_file, this));
-	
+
     handler->process();
 
     return 0;
@@ -677,9 +876,9 @@ int download(const std::string & source, CURL* handle,
             CURLcode error;
             std::cout << "Downloading " << source
                       << " to " << dest << std::endl;
-            
+
             fp = fopen(dest.c_str(), "wb");
-            
+
             if(fp == NULL) {
                 std::cout << " File Error " << strerror(errno) << std::endl;
                 dest = "";
@@ -719,7 +918,7 @@ void TestSuite::walk_tests(xmlNode * testsuite, CURL* handle,
     xmlNode *test = testsuite->children;
 
     while(test) {
-        if((test->type == XML_ELEMENT_NODE) 
+        if((test->type == XML_ELEMENT_NODE)
            && (strcmp((const char*)(test->name), "test") == 0)) {
             xmlNode * childrens = test->children;
             while(childrens) {
@@ -732,25 +931,23 @@ void TestSuite::walk_tests(xmlNode * testsuite, CURL* handle,
         }
         test = test->next;
     }
-    
-    std::map<std::string, Test::Ptr>::const_iterator iter(m_tests.begin());
-    for( ; iter != m_tests.end(); iter++) {
-        std::string n = iter->first;
+
+    for(const auto & elem : m_tests) {
+        std::string n = elem.first;
         std::string dest;
-        int ret = download(iter->second->source(), handle, download_dir, dest);
+        int ret = download(elem.second->source(), handle, download_dir, dest);
 
         if(ret == 0 && !dest.empty()) {
             xmlNode * test2 = NULL;
-            std::map<std::string, xmlNode *>::iterator iter2 
-                = overrides.find(n);
-            if (iter2 != overrides.end()) {
+            auto iter2 = overrides.find(n);
+            if (iter2 != overrides.cend()) {
                 test2 = iter2->second;
             }
             else {
                 test2 = xmlNewNode(NULL, (const xmlChar*)"test");
                 xmlAddChild(testsuite, test2);
-                xmlNode *name_node = xmlNewTextChild(test2, NULL, 
-                                                     (const xmlChar*)"name", 
+                xmlNode *name_node = xmlNewTextChild(test2, NULL,
+                                                     (const xmlChar*)"name",
                                                      (const xmlChar*)n.c_str());
                 xmlAddChild(test2, name_node);
             }
@@ -779,7 +976,7 @@ int curl_write_function(void *buffer, size_t size, size_t nmemb, void *stream)
 
     return (int) w;
 }
-#endif 
+#endif
 
 }
 
@@ -792,7 +989,7 @@ int TestSuite::bootstrap(const std::string & overrides_file,
     CURL *handle;
 
     handle = curl_easy_init();
-    curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, curl_write_function);    
+    curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, curl_write_function);
 
     doc = xmlReadFile(overrides_file.c_str(), NULL, 0);
     xmlNode *root_element;
@@ -801,7 +998,7 @@ int TestSuite::bootstrap(const std::string & overrides_file,
     }
     else {
         doc = xmlNewDoc((const xmlChar *)"1.0");
-        root_element = xmlNewDocNode(doc, 
+        root_element = xmlNewDocNode(doc,
                                      NULL, (xmlChar*)"testsuite", NULL);
         xmlDocSetRootElement(doc, root_element);
     }
@@ -830,9 +1027,8 @@ int TestSuite::bootstrap(const std::string & /*overrides_file*/,
 int TestSuite::run_all()
 {
     int failures = 0;
-    std::map<std::string, Test::Ptr>::iterator iter(m_tests.begin());
-    for( ; iter != m_tests.end(); ++iter) {
-        failures += iter->second->run();
+    for(const auto & elem : m_tests) {
+        failures += elem.second->run();
     }
     return failures;
 }
